#include "utils.h"
#include "abcplayer.h"


AbcPlayer::AbcPlayer()
{
    end_time = 0;
}

void AbcPlayer::parse(char const *string, int timestep)
{
    char const *s = string;

    int state = S_SKIP;

    int curnote = 0;
    int curlen = 1;
    int modifiers = 0;
    int vol;

    int time = 0;

    while (*s)
    {
        switch(state)
        {
            case S_SKIP:
                curnote = note(*s);

                if (curnote >= 0)
                    state = S_NOTE;
                else
                    modifiers |= modifier(*s);

                if (*s == 'z' || *s == 'Z')
                    state = S_REST;

                break;

            case S_REST:
                    if (*s - '0'> 0 && *s - '0'< 10)
                    {
                        curlen = *s - '0';
                    }

                    time += curlen * timestep;

                    curlen = 1;
                    modifiers = 0;
                    state = S_SKIP;
                    break;
            case S_NOTE:
                if (*s == ',')
                {
                    curnote -=12;
                }
                else if (*s == '\'')
                {
                    curnote +=12;
                }
                else
                {
                    if (*s - '0'> 0 && *s - '0'< 10)
                    {
                        curlen = *s - '0';
                    }

                    if (modifiers & M_SHARP)
                        curnote++;
                        
                    if (modifiers & M_FLAT)
                        curnote--;
                        
                    vol = 128;
                    
                    if (modifiers & M_LOUD)
                        vol = 255;

                    add_event_absolute(EVENT_NOTE_ON, time, curnote, vol);
                    add_event_absolute(EVENT_NOTE_OFF, time + (int)(curlen * timestep * .75), curnote);
                    time += curlen * timestep;
//                    printf("abcplayer parser time now is %d\n", time);
                    curlen = 1;
                    modifiers = 0;
                    state = S_SKIP;
                }

                break;
        }
        s++;
    }

    if (time > end_time)
        end_time = time;
}


int AbcPlayer::note(int n)
{
    static int notes[] = {'C', 'D', 'E', 'F', 'G', 'A', 'B', 'c', 'd', 'e', 'f', 'g', 'a', 'b'};
    static int notevalues[] = {27,29,31, 32,34,36,38, 39,41,43, 44,46,48,50};
    int i;

    for (i = 0; i< (int)(sizeof(notes)/sizeof(int)); i++)
    {
        if (n == notes[i])
            return notevalues[i];
    }

    return -1;
}

int AbcPlayer::modifier(int n)
{
    if (n == '^')
        return M_SHARP;

    if (n == '_')
        return M_FLAT;

    if (n == '!')
        return M_LOUD;

    return 0;
}

void AbcPlayer::prepare_for_playing()
{
    add_event_absolute(EVENT_END_SONG, end_time);
    ready_to_play = -1;
}


void AbcPlayer::clear()
{
    stop();
    EventList::clear();

    ready_to_play = 0;
    end_time = 0;
}


void AbcPlayer::parse(Song *s, int timestep)
{
    int i;
    for (i = 0; i< s->get_nr_voices(); i++)
        parse(s->get_voice(i), s->get_speed());

    prepare_for_playing();
}
