#include "utils.h"
#include "math.h"
#include "bank.h"

Bank::Bank(int n, Synth const * basetone)
{
    int i;
    num_tones = n;
    synths = new Synth *[NUM_VOICES];
    tones = new double[num_tones];
    notemap = new int[num_tones];
    synthmap = new int[num_tones];

    int synthbuftime = SYNTH_BUF * 1000 / SYNTH_FREQ;

    int wtime = synthbuftime / NUM_VOICES;


    for (i = 0; i < NUM_VOICES; i++)
    {
        synths[i] = new Synth(1, basetone);
        rest(wtime); // to prevent all buffers needing updating at once
    }
    for (i = 0 ; i < num_tones;i++)
    {
        tones[i] = exp((i/12.0) * log(2));
        notemap[i] = 0;
        synthmap[i] = -1;
    }
    
}

Bank::~Bank()
{
    int i;
    for (i=0;i<NUM_VOICES;i++)
    {
        delete synths[i];
    }

    delete [] synths;
    delete [] tones;
    delete [] synthmap;
    delete [] notemap;
}

int Bank::find_free_synth()
{
    int i;
    static int count = 0;

    for (i = 0; i < NUM_VOICES; i++)
    {
        if (synths[i]->get_state() == STATE_STOPPED)
            return i;
    }

    for (i = 0; i < NUM_VOICES; i++)
    {
        if (synths[i]->get_state() == STATE_STOPPING)
            return i;
    }

    count++;
    count %= NUM_VOICES;
    
    return count;

    
}

void Bank::note_on(int note, double volume)
{
    if (note < 0 || note >= num_tones)
        return;
        
    if (!notemap[note])
    {
        synthmap[note] = find_free_synth();
        synths[synthmap[note]]->start(tones[note], volume, vibtime, vibdepth);
    }
    notemap[note]++;
}

void Bank::note_off(int note)
{
    if (note < 0 || note >= num_tones)
        return;


    if (!notemap[note])
        return;
        
    notemap[note]--;
    
    if (!notemap[note])
    {
        synths[synthmap[note]]->stop();
        synthmap[note] = -1;
    }
}

void Bank::update()
{
    int i;
    for (i = 0; i < NUM_VOICES; i++)
    {
//        if (synths[i]->get_state())
//        {
            synths[i]->update();
//        }
    }
}

void Bank::set_vibrato(int vibt, double vibd)
{
    vibdepth = (int)(vibd * synths[0]->get_synthfreq());
    vibtime = vibt;


}
