#include <stdio.h>
#include "eventlist.h"

static int gtime;

static void timer()
{
    gtime++;
}
END_OF_FUNCTION(timer);


void reset_timer()
{
    gtime = 0;
}


EventList::EventList()
{
    last = list = NULL;

    if (!installed_timer)
    {
        LOCK_VARIABLE(gtime);
        LOCK_FUNCTION(timer);
        install_int(timer, 1);
        installed_timer = 1;
    }

    mains = NULL;
    
}


int EventList::installed_timer = 0;


void EventList::process()
{
    EVENT *tmp;
    
    if (list)
    {
        while (list && list->time < gtime)
        {
            tmp = list;
            list = list->next;
            
            if (list)
                list->prev = NULL;
                

            if (last && list == last)
                last->prev = NULL;
                
            if (!list)
                last = NULL;

            execute_event(tmp);
            delete_event(tmp);
        }
    }
}


void EventList::execute_event(EVENT *e)
{
    switch(e->event)
    {
        case EVENT_NOTE_ON:
            b->note_on(e->data1, e->data2/255.0);
            break;
        case EVENT_NOTE_OFF:
            b->note_off(e->data1);
            break;
        case EVENT_NOTIFY:
            e->callback(e->event, e->data1, e->data2, e->otherdata);
            break;
        default:
            break;
    }

    if (mains && mains->do_animation)
        mains->state_changed();
}

void EventList::add_event(int e, int time, int data1, int data2, void (*callback)(int, int, int, void *), void *otherdata)
{

    EVENT *event = new_event();


    event->event = e;
    event->time = gtime + time;
    event->data1 = data1;
    event->data2 = data2;
    event->callback = callback;
    event->otherdata = otherdata;


    insert_event(event);

//    dumplist();
}

void EventList::add_event(EVENT *e)
{

    EVENT *event = new_event();


    event->event = e->event;
    event->time = gtime + e->time;
    event->data1 = e->data1;
    event->data2 = e->data2;
    event->callback = e->callback;
    event->otherdata = e->otherdata;

    insert_event(event);

//    dumplist();

    
}


void EventList::add_event_absolute(int e, int time, int data1, int data2, void (*callback)(int, int, int, void *), void *otherdata)
{

    EVENT *event = new_event();


    event->event = e;
    event->time = time;
    event->data1 = data1;
    event->data2 = data2;
    event->callback = callback;
    event->otherdata = otherdata;


    insert_event(event);

//    dumplist();
}

void EventList::add_event_absolute(EVENT *e)
{

    EVENT *event = new_event();


    event->event = e->event;
    event->time = e->time;
    event->data1 = e->data1;
    event->data2 = e->data2;
    event->callback = e->callback;
    event->otherdata = e->otherdata;

    insert_event(event);

//    dumplist();

    
}



void EventList::insert_event(EVENT *event)
{
    EVENT *i;

    
    // list is empty
    if (!last)
    {
//        printf(" empty list\n");
        last = list = event;
        list->next = list->prev = NULL;
        return;
    }

    // tack it on at the back
    if (event->time >= last->time)
    {
        event->prev = last;
        last->next = event;
        event->next = NULL;
        last = event;
        return;
    }

    // tack it on at the front
    if (event->time <= list->time)
    {
        event->prev = NULL;
        list->prev = event;
        event->next = list;
        list = event;
        return;
    }


    // insert it
    i = last;


    ASSERT(i->time > event->time);
    
    while (i->time > event->time)
    {
        ASSERT(i);

        event->next = i;
        i = i->prev;
    }


    event->prev = event->next->prev;
    event->next->prev = event;

    if (event->prev)
        event->prev->next = event;


}



EVENT *EventList::new_event()
{
    EVENT *ret;
    
    if (freelist)
    {
        ret = freelist;
        freelist = freelist->next;
    }
    else
    {
        ret = (EVENT *)malloc(sizeof(EVENT));
    }
    
    return ret;
    
}

void EventList::delete_event(EVENT *e)
{
    e->next = freelist;

    freelist = e;
}

void EventList::clear()
{

    if (last)
    {
        last->next = freelist;

        freelist = list;

        last = list = NULL;
    }

    ASSERT(!list);
    ASSERT(!last);
}




EVENT *EventList::freelist = NULL;


void EventList::set_bank(Bank *_b)
{
    b = _b;
}


void EventList::dumplist()
{
    EVENT *i = list;

    while(i)
    {
//        printf("    %d \n", i->event);
        i = i->next;
    }

    
}
