#include "utils.h"
#include "keyb.h"


KeyBoard::KeyBoard()
{
    int i;
    for (i = 0; i < KEY_MAX; i++)
    {
        lastmap[i] = 0;
        tonemap[i] = 0;
    }
    for (i=0; i<NUM_RECORDERS; i++)
    {
        recorders[i] = new Recorder;
        recorder_states[i] = RECORDER_EMPTY;
    }

    b = 0;
}


KeyBoard::~KeyBoard()
{
    int i;
    for (i=0; i<NUM_RECORDERS; i++)
    {
        delete recorders[i];
    }
}

void KeyBoard::bind_key(int key, COMMANDS command, int data)
{
    tonemap[key] = command + (data << 8);
}



void KeyBoard::set_bank(Bank *bank)
{
    b = bank;
}

void KeyBoard::process()
{
    int i;
    for (i = 0; i < KEY_MAX; i++)
    {
        if (key[i] && !lastmap[i])
            key_on(i);

        if (!key[i] && lastmap[i])
            key_off(i);

        lastmap[i] = key[i];
    }

}

void KeyBoard::key_on(int i)
{
    int command = tonemap[i] & 0xFF;
    int data = tonemap[i] >> 8;


    switch(command)
    {
        case C_NOTE:
            b->note_on(data + transpose, 1.0);
            record(EVENT_NOTE_ON, data, 255);
            break;
        case C_RECORDER_TOGGLE:
            toggle_recorder(data);
            break;
        case C_TRANSPOSE:
            transpose = data;
            break;
        case C_RECORDER_CLEAR:
            clear_recorder(data);
            break;
        case C_JUKEBOX_PLAY:
            j->play();
            break;
        case C_JUKEBOX_STOP:
            j->stop();
            break;
        case C_JUKEBOX_NEXT:
            j->next();
            break;
        case C_JUKEBOX_PREV:
            j->prev();
            break;
        case C_VIBRATO:
            b->set_vibrato(50, .02);
            break;
        case C_TOGGLE_ANIMATION:
            toggle_animation();
            break;
            
        default:
            break;
    }
    mains->state_changed();
}

void KeyBoard::key_off(int i)
{
    int command = tonemap[i] & 0xFF;
    int data = tonemap[i] >> 8;

    switch(command)
    {
        case C_NOTE:
            b->note_off(data + transpose);
            record(EVENT_NOTE_OFF, data);
            break;
        case C_TRANSPOSE:
            transpose = 0;
            break;
        case C_VIBRATO:
            b->set_vibrato(0, 0);
        default:
            break;
    }
    mains->state_changed();
}

void KeyBoard::set_eventlist(EventList *l)
{
    int i;
    for (i = 0; i < NUM_RECORDERS; i++)
        recorders[i]->set_output(l);
}

void KeyBoard::clear_recorder(int r)
{
    recorder_states[r] =  RECORDER_EMPTY;
    recorders[r]->clear();
}

void KeyBoard::toggle_recorder(int r)
{

    switch(recorder_states[r])
    {
        case RECORDER_EMPTY:
            recorder_states[r] =  RECORDER_RECORDING;
            break;
        case RECORDER_RECORDING:
            recorder_states[r] =  RECORDER_STOPPED;
            recorders[r]->recording_finished();
            if (!recorders[r]->has_recorded())
                recorder_states[r] =  RECORDER_EMPTY;
            break;
        case RECORDER_STOPPED:
            recorder_states[r] =  RECORDER_PLAYING;
            recorders[r]->play(0);
            break;
        case RECORDER_PLAYING:
            recorder_states[r] =  RECORDER_STOPPED;
            recorders[r]->stop();
            break;
        
    }
}


void KeyBoard::record(int event, int data1, int data2)
{
    int i;
    for (i = 0; i< NUM_RECORDERS; i++)
    {
        if (recorder_states[i] == RECORDER_RECORDING)
        {
            recorders[i]->event(event, data1, data2);
        }
    }
}

void KeyBoard::toggle_animation()
{
    mains->do_animation = !mains->do_animation;
    
}

