#ifndef _KEYB_H_
#define _KEYB_H_

#include "bank.h"
#include "recorder.h"
#include "jukebox.h"

#define NUM_RECORDERS 4


typedef enum RECORDER_STATES
{
    RECORDER_EMPTY,
    RECORDER_RECORDING,
    RECORDER_STOPPED,
    RECORDER_PLAYING
};


typedef enum COMMANDS
{
    C_NOTE,
    C_TRANSPOSE,
    C_CLEAR_KEYBOARD,
    C_RECORDER_TOGGLE,
    C_RECORDER_CLEAR,
    C_JUKEBOX_PLAY,
    C_JUKEBOX_STOP,
    C_JUKEBOX_NEXT,
    C_JUKEBOX_PREV,
    C_VIBRATO,
    C_TOGGLE_ANIMATION,
    C_LAST
};

class MainScreen;

class KeyBoard
{
    public:
        KeyBoard();
        ~KeyBoard();
        
        void bind_key(int key, COMMANDS command, int data = 0);
        void set_bank(Bank *b);
        void set_jukebox(JukeBox *_j) { j = _j;};
        void set_eventlist(EventList *l);
        void set_mainscreen(MainScreen *m) {mains = m;};
        void process();
        void key_on(int i);
        void key_off(int i);
        void toggle_recorder(int recorder);
        void toggle_animation();
        void clear_recorder(int r);
        void record(int event, int data1 = 0, int data2 = 0);


        JukeBox *get_jukebox() {return j;};
        Bank *get_bank() { return b;};

        int get_num_recorders() { return NUM_RECORDERS;};
        int get_recorder_state(int i) { return recorder_states[i];};
        
    private:
        int lastmap[KEY_MAX];
        int tonemap[KEY_MAX];
        Bank *b;
        JukeBox *j;
        Recorder *recorders[NUM_RECORDERS];
        int recorder_states[NUM_RECORDERS];

        int transpose;
        MainScreen *mains;

        
};

#include "mainscreen.h"

#endif
