#include <allegro.h>
#include "keyb.h"
#include "eventlist.h"
#include "abcplayer.h"
#include "gfxbase.h"
#include "mainscreen.h"
#include "songs.h"


void bind_keys(KeyBoard *keyb)
{
    keyb->bind_key(KEY_LSHIFT, C_TRANSPOSE, -24);
    keyb->bind_key(KEY_Z, C_NOTE, 27);
    keyb->bind_key(KEY_S, C_NOTE, 28);
    keyb->bind_key(KEY_X, C_NOTE, 29);
    keyb->bind_key(KEY_D, C_NOTE, 30);
    keyb->bind_key(KEY_C, C_NOTE, 31);
    keyb->bind_key(KEY_V, C_NOTE, 32);
    keyb->bind_key(KEY_G, C_NOTE, 33);
    keyb->bind_key(KEY_B, C_NOTE, 34);
    keyb->bind_key(KEY_H, C_NOTE, 35);
    keyb->bind_key(KEY_N, C_NOTE, 36);
    keyb->bind_key(KEY_J, C_NOTE, 37);
    keyb->bind_key(KEY_M, C_NOTE, 38);

    keyb->bind_key(KEY_COMMA, C_NOTE, 39);
    keyb->bind_key(KEY_L, C_NOTE, 40);
    keyb->bind_key(KEY_STOP, C_NOTE, 41);
    keyb->bind_key(KEY_COLON, C_NOTE, 42);
    keyb->bind_key(KEY_SLASH, C_NOTE, 43);



    keyb->bind_key(KEY_Q, C_NOTE, 39);
    keyb->bind_key(KEY_2, C_NOTE, 40);
    keyb->bind_key(KEY_W, C_NOTE, 41);
    keyb->bind_key(KEY_3, C_NOTE, 42);
    keyb->bind_key(KEY_E, C_NOTE, 43);
    keyb->bind_key(KEY_R, C_NOTE, 44);
    keyb->bind_key(KEY_5, C_NOTE, 45);
    keyb->bind_key(KEY_T, C_NOTE, 46);
    keyb->bind_key(KEY_6, C_NOTE, 47);
    keyb->bind_key(KEY_Y, C_NOTE, 48);
    keyb->bind_key(KEY_7, C_NOTE, 49);
    keyb->bind_key(KEY_U, C_NOTE, 50);

    keyb->bind_key(KEY_I, C_NOTE, 51);
    keyb->bind_key(KEY_9, C_NOTE, 52);
    keyb->bind_key(KEY_O, C_NOTE, 53);
    keyb->bind_key(KEY_0, C_NOTE, 54);
    keyb->bind_key(KEY_P, C_NOTE, 55);
    keyb->bind_key(KEY_OPENBRACE, C_NOTE, 56);


    keyb->bind_key(KEY_F1, C_RECORDER_TOGGLE, 0);
    keyb->bind_key(KEY_F2, C_RECORDER_TOGGLE, 1);
    keyb->bind_key(KEY_F3, C_RECORDER_TOGGLE, 2);
    keyb->bind_key(KEY_F4, C_RECORDER_TOGGLE, 3);
    keyb->bind_key(KEY_F5, C_RECORDER_CLEAR, 0);
    keyb->bind_key(KEY_F6, C_RECORDER_CLEAR, 1);
    keyb->bind_key(KEY_F7, C_RECORDER_CLEAR, 2);
    keyb->bind_key(KEY_F8, C_RECORDER_CLEAR, 3);

    keyb->bind_key(KEY_F9, C_JUKEBOX_PLAY);
    keyb->bind_key(KEY_F10, C_JUKEBOX_STOP);
    keyb->bind_key(KEY_F11, C_JUKEBOX_NEXT);
    keyb->bind_key(KEY_F12, C_JUKEBOX_PREV);

    keyb->bind_key(KEY_SPACE, C_VIBRATO);
    keyb->bind_key(KEY_TAB, C_TOGGLE_ANIMATION);
}


int main(int argc, char **argv)
{
    allegro_init();
    

    install_keyboard();

    reserve_voices(NUM_VOICES,0);
    
    if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL))
    {
        allegro_message("no sound!");
        exit(1);
    }


    setup_gfx();

    rest(1000);

    // change this to get another sound


    Synth *base = new Synth(4); // create a new synth, the parameter tells
                                // how many sines we want to add up to create the sound

    // add the sines: don't add more than you allocated above.
    //the parameters are:
    //  frequency - the frequency in Hz of the sine
    //  attack - how long it takes from starting to reach full volume (sec)
    //  decay  - how long it takes to decay from full volume to sustain volume (sec)
    //  sustain - the sustain volume, i.e. the volume you get in the end when you
    //            hold a not for a long time (0-1)
    //  volume  - the volume relative to the other sines (this scales both the full volume and
    //            the sustain volume) (0-1)
    //  tips:   440Hz and multiples/divisors thereof give an A, and the program
    //          assumes the base synth produces an A, but it is also fun to
    //          add some totally out of tune sines together to get a real dirty
    //          'hammond-organ' sound.
    
    base->add_sine(55, .05, 10,  0, 1);
    base->add_sine(220, .05, 10, 0, .4);
    base->add_sine(440, .05, 1, 0, .2);
    base->add_sine(880, .05, 1, 0, .2);


    ///

    Bank *bank = new Bank(60, base);

    KeyBoard *keyb = new KeyBoard();

    reset_timer();

    bind_keys(keyb);

    keyb->set_bank(bank);


    EventList *eventlist = new EventList;

    eventlist->set_bank(bank);

    keyb->set_eventlist(eventlist);


    SongCollection *songs = new SongCollection();
    JukeBox *jukebox = new JukeBox(songs, eventlist);
    keyb->set_jukebox(jukebox);

    MainScreen *m = new MainScreen();

    m->set_keyboard(keyb);
    keyb->set_mainscreen(m);
    eventlist->set_mainscreen(m);

    while (!key[KEY_ESC])
    {
        keyb->process();

        bank->update();
        eventlist->process();

        m->draw();
    }
    


    return 0;
}
END_OF_MAIN();
