#include "mainscreen.h"
#include "gfxbase.h"
#include "keyb.h"


#define JUKEBOXL    600
#define JUKEBOXW    180
#define JUKEBOXT     20
#define JUKEBOXH    200

#define PIANOT  500
#define PIANOB 20

#define RECW 100
#define RECH 20
#define RECB 20

void MainScreen::draw()
{
    if (!dirty)
        return;

    dirty = 0;


    clear_to_color(screenbuffer, makecol(100,120,110));



    if (do_animation)
        draw_piano();


    draw_recorders();
        
    draw_jukebox();
    refresh();
    
}

void MainScreen::draw_piano()
{
    Bank *b = keyboard->get_bank();
    int nrkeys = b->get_num_tones();
    int i;
    int keywidth = (SCREEN_W - 2*PIANOB) / nrkeys;
    int color;
    int blackkey_len = ((screenbuffer->h - PIANOT) - PIANOB) * 2 / 3;
    int left, right;
    static int black[] = {0, 1, 0, 0, 1, 0,1, 0,0,1, 0, 1};

    for (i = 0; i< nrkeys;i++)
    {
        if (b->note_pressed(i))
            color = black[i % 12] ? makecol(100,0,0) : makecol(255,155,155);
        else
            color = black[i % 12] ? makecol(0,0,0) : makecol(255,255,255);

        
        rectfill
        (
            screenbuffer,
            PIANOB + i * keywidth,
            PIANOT,
            PIANOB + i * keywidth + keywidth - 1,
            PIANOT + blackkey_len,
            color
        );

        right = left = 0;

        if (!black[i % 12])
        {
            if (black[(i + 11) % 12])
                left = keywidth / 2;
                
            if (black[(i + 1) % 12])
                right = keywidth / 2;

            rectfill
            (
                screenbuffer,
                PIANOB + i * keywidth - left,
                PIANOT + blackkey_len,
                PIANOB + i * keywidth + keywidth + right - 1,
                screenbuffer->h - PIANOB,
                color
            );
                
        }
        
    }
    

}

void MainScreen::draw_jukebox()
{
    int i;
    JukeBox *j = keyboard->get_jukebox();
    int cursong = j->get_cur_song();


    
    set_clip(screenbuffer, JUKEBOXL, JUKEBOXT, JUKEBOXL+JUKEBOXW, JUKEBOXT+JUKEBOXH);
    rectfill(screenbuffer, JUKEBOXL, JUKEBOXT+20, JUKEBOXL+JUKEBOXW, JUKEBOXT+JUKEBOXH, makecol(100,100,100));

    text_mode(makecol(100,120,110));
    if (j->is_playing())
        textout(screenbuffer, font, "JukeBox" , JUKEBOXL + 2, JUKEBOXT+2, makecol(200,0,0));
    else
        textout(screenbuffer, font, "JukeBox" , JUKEBOXL + 2, JUKEBOXT+2, makecol(100,0,0));
        
    for (i =0 ; i < j->get_nr_songs();i++)
    {
        if (i == cursong)
        {
            text_mode(makecol(100,150,140));
            textout(screenbuffer, font, j->get_title(i) ,JUKEBOXL + 2, JUKEBOXT+22+ text_height(font) * i, makecol(0,0,0));
        }
        else
        {
            text_mode(makecol(100,100,100));
            textout(screenbuffer, font, j->get_title(i), JUKEBOXL + 2, JUKEBOXT+22+ text_height(font) * i, makecol(100,150,140));
        }
        
    }
    
    set_clip(screenbuffer, 0,0, screenbuffer->w - 1, screenbuffer->h - 1);

}


void MainScreen::draw_recorders()
{
    int i;
    int num_recorders = keyboard->get_num_recorders();

    int state;
    int color = 0;
    char *text;
    
    for (i = 0; i< num_recorders; i++)
    {
        state = keyboard->get_recorder_state(i);

        switch(state)
        {
            default:
            // fall through
            case RECORDER_EMPTY:
             color = makecol( 100,100,100);
             text = "empty";
             break;
            case RECORDER_RECORDING:
             color = makecol( 255,0,0);
             text = "rec";
             break;
            case RECORDER_STOPPED:
             color = makecol(0,100,0);
             text = "stopped";
             break;
            case RECORDER_PLAYING:
             color = makecol(0,255,0);
             text = "playing";
             break;
        }

        rectfill(screenbuffer, RECB + i * (RECW + RECB), RECB, RECB + i * (RECW + RECB) + RECW, RECB + RECH, color);
        text_mode(-1);
        textout(screenbuffer, font, text, RECB + i * (RECW + RECB) + 2, RECB + 5, makecol(0,0,0));
        
    }
    
}
