#ifndef _SYNTH_H_
#define _SYNTH_H_
#include <allegro.h>
#include "math.h"

#ifndef M_PI
#define M_PI 3.141592654
#endif


#define SINTABSIZE 10000
#define SYNTH_FREQ 44100
#define SYNTH_BUF 1024
#define STOPPING_RAMP 300 // in millisecs
#define STOPPING_BUFFERS  ((STOPPING_RAMP *  SYNTH_FREQ) / (SYNTH_BUF * 1000))


class SinTab
{
    public:
        SinTab();
        double sin(double s);
    private:
        double sintab[100000];
};

class Sine
{
    public:
        Sine(double freq, double attack , double decay, double sustain);
        Sine(double relfreq, Sine const * other);
        ~Sine();
        int render(int pos, int destfreq, int destbits);

        
    private:
        int gtime ;

        double freq, attack, decay, sustain;

        static int refcount;
        static SinTab *sintab;
        
        
};


typedef enum
{
    STATE_STOPPED = 0,
    STATE_PLAYING,
    STATE_STOPPING
} SYNTH_STATES;

class Synth
{
    public:
        Synth(int nr_sines);
        Synth(double relfreq, Synth const * other);
        ~Synth();
        void add_sine(double freq, double attack, double decay, double sustain, double vol);

        void start(double relfreq, double volume, int _vibrato_time = 0, int _vibrato_depth = 0);
        int update();
        void stop();

        int get_state() { return state;};
        int get_synthfreq();
        
    private:
        int nr_sines, max_nr_sines;
        int count;
        Sine **sines;
        double *volumes;

        double relfreq;
        AUDIOSTREAM *stream;

        int state;
        int buffers_to_go;

        int vibtime, lastvibtime, vibdepth, vibdir;

        double volume;
        
};

#endif
